"use strict";
//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIExtractorFormForUser = void 0;
var react_1 = __importDefault(require("react"));
var action_button_1 = __importDefault(require("../../controls/action-button"));
var primeobjects_helper_util_1 = require("primeobjects-helper-util");
var primeobjects_ui_web_basic_1 = require("primeobjects-ui-web-basic");
var ai_extractor_util_1 = require("./ai-extractor-util");
var AIExtractorFormForUser = function (props) {
    var entity = props.entity, hideTabs = props.hideTabs, onGoBack = props.onGoBack, record = props.record;
    var FormWrapper = props.FormWrapper;
    var getGeneralForm = function (customProps) {
        var solution = customProps.solution, isError = customProps.isError, isDoing = customProps.isDoing, version = customProps.version, disabled = customProps.disabled, onStartAPICall = customProps.onStartAPICall, onSuccessAPICall = customProps.onSuccessAPICall, toChangeRecord = customProps.toChangeRecord, onErrorAPICall = customProps.onErrorAPICall, onError = customProps.onError;
        var getRows = function (record) {
            if ((0, primeobjects_helper_util_1.libTrackIsOn)())
                console.log({ record: record });
            var updateRecord = function (newRecord) {
                record = newRecord;
                toChangeRecord(record);
            };
            var clearAllError = function () {
                onError("ai", false);
            };
            var onClickSubmitButton = function () {
                (0, ai_extractor_util_1.startAIExtractorJob)(solution, record, "task", updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall);
            };
            return [
                ((0, primeobjects_helper_util_1.isNonEmptyString)(record.name) ||
                    (0, primeobjects_helper_util_1.isNonEmptyString)(record.description)) && {
                    fields: [
                        {
                            name: "introduction",
                            type: "container",
                            content: (react_1.default.createElement("div", { className: "w-full" },
                                (0, primeobjects_helper_util_1.isNonEmptyString)(record.name) && (react_1.default.createElement("div", { className: "font-medium text-base mb-1" }, record.name)),
                                (0, primeobjects_helper_util_1.isNonEmptyString)(record.description) && (react_1.default.createElement("div", { className: "text-sm" }, record.description))))
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "taskFileUrl",
                            label: "Upload a file to AI extractor",
                            alwaysShowLabel: true,
                            fileType: "Extract",
                            subNote: "Supported format: pdf, jpeg and jpg",
                            type: "upload",
                            disabled: isDoing("ai"),
                            onChange: function (newRecord) {
                                clearAllError();
                                var fileName = (0, ai_extractor_util_1.getAIExtractorFileName)("task", newRecord);
                                delete newRecord.taskJobId;
                                delete newRecord.taskResult;
                                if (fileName) {
                                    return __assign(__assign({}, newRecord), { fileName: fileName });
                                }
                                return newRecord;
                            }
                        }
                    ]
                },
                !isDoing("ai") &&
                    (0, primeobjects_helper_util_1.isNonEmptyString)(record.taskFileUrl) && {
                    fields: [
                        {
                            name: "ai",
                            colStyle: {
                                alignItems: "flex-end",
                                flex: "none"
                            },
                            content: (react_1.default.createElement("div", { className: "flex flex-row" },
                                react_1.default.createElement(action_button_1.default, { id: "ai-submit-button", wrapperStyle: { marginLeft: 0 }, onClick: onClickSubmitButton, wrapperClassName: "w-32 bg-sky-400 hover:bg-sky-500 justify-center", text: "Submit to AI", textColor: primeobjects_helper_util_1.COLORS.WHITE }))),
                            type: "custom"
                        }
                    ]
                },
                isError("ai") && {
                    fields: [
                        {
                            name: "aiFailed",
                            type: "alert-error",
                            line: "Sorry, we failed to process the uploaded file. Please try again later."
                        }
                    ]
                },
                isDoing("ai") && {
                    fields: [
                        {
                            name: "aiDoing",
                            content: (react_1.default.createElement("div", { className: "w-full flex flex-row" },
                                react_1.default.createElement(primeobjects_ui_web_basic_1.SVG, { src: "/icons/platform/loading.svg", className: "spinner", style: { width: 22, height: 22 }, color: primeobjects_helper_util_1.COLORS.SKY["600"] }),
                                react_1.default.createElement("span", { className: "pl-2 text-sky-600" }, "Processing the file ..."))),
                            type: "custom"
                        }
                    ]
                },
                record.taskResult && {
                    fields: [
                        {
                            name: "taskResultForFieldsHTML",
                            type: "html",
                            value: "<pre><code class=\"language-json\">".concat(JSON.stringify(record.taskResult, null, 4), "</code></pre>"),
                            disabled: true,
                            useFieldValue: true
                        }
                    ]
                }
            ];
        };
        return {
            id: "general",
            tooltip: "General Form",
            iconSrc: "/icons/platform/form.svg",
            getRows: getRows,
            version: version,
            disabled: disabled == true
        };
    };
    return (react_1.default.createElement(FormWrapper, __assign({}, props, { tabs: [getGeneralForm], entity: entity, record: record, hideSaveButton: true, hideActionButton: true, hideTabs: hideTabs, onGoBack: onGoBack })));
};
exports.AIExtractorFormForUser = AIExtractorFormForUser;
//# sourceMappingURL=ai-extractor-for-user.js.map