//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { getWebLocation, isGuid, setCache, getCache, isNonEmptyString, libTrackIsOn } from "primeobjects-helper-util";
import { callApi } from "../../context/auth-call-api";
import { filter, delay, forOwn, isObject, find, map, endsWith } from "lodash";
import { getApiEndpoint } from "../../util";
export var getAIExtractorS3FileName = function (requestType, record) {
    var file = getWebLocation(record["".concat(requestType, "FileUrl")]);
    return file && (file === null || file === void 0 ? void 0 : file.path);
};
export var getAIExtractorFileName = function (requestType, record) {
    var s3FileName = getAIExtractorS3FileName(requestType, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if (isGuid(fileName.split("_")[0])) {
            fileName = fileName.replace("".concat(fileName.split("_")[0], "_"), "");
        }
        return fileName;
    }
    return undefined;
};
export var getAIExtractorFileId = function (requestType, record) {
    var s3FileName = getAIExtractorS3FileName(requestType, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if (isGuid(fileName.split("_")[0])) {
            return fileName.split("_")[0];
        }
    }
    return undefined;
};
export var processAIExtractorResult = function (requestType, record) {
    if (!isObject(record.mapping))
        return {};
    var result = {};
    forOwn(record.mapping, function (value, key) {
        //console.log({ key });
        var keyInfo = key.split("_");
        if (keyInfo.length == 2) {
            var type = keyInfo[1];
            var key_1 = keyInfo[0];
            switch (type) {
                case "LABEL": {
                    var fieldValue = find(record["".concat(requestType, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            endsWith(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return isNonEmptyString(f.label);
                        }
                        return f.key == key_1 && isNonEmptyString(f.label);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.label;
                    }
                    break;
                }
                case "VALUE": {
                    var fieldValue = find(record["".concat(requestType, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            endsWith(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return isNonEmptyString(f.value);
                        }
                        return f.key == key_1 && isNonEmptyString(f.value);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.value;
                    }
                    break;
                }
                default: {
                    //COLUMN
                    var cellPropName_1 = "cell".concat(parseInt(type.replace("C", "")));
                    //console.log({ cellPropName, key });
                    var fieldValue = find(record["".concat(requestType, "ResultForRows")], function (row) {
                        if (!isNonEmptyString(row[cellPropName_1]))
                            return false;
                        if (row.key == key_1)
                            return true;
                        //try all fields
                        for (var i = 0; i < row.cellsCount; i++) {
                            // console.log({
                            //     cellKey: `cell${i + 1}Key`,
                            //     key
                            // });
                            if (row["cell".concat(i + 1, "Key")] == key_1)
                                return true;
                        }
                        return false;
                    });
                    if (fieldValue) {
                        result[value] = fieldValue[cellPropName_1];
                    }
                    break;
                }
            }
        }
    });
    return result;
};
export var processAIExtractorJob = function (solution, record, requestType, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount) {
    if (libTrackIsOn())
        console.log({ onProcessJob: jobId });
    onStartAPICall("ai");
    delay(function () {
        var retryCountKey = "processAIExtractorJob".concat(jobId);
        var retryCount = getCache("retryCountKey", 0) + 1;
        callApi(solution, getApiEndpoint(solution, "ai", "process-extract-text"), 
        //"http://localhost:3000/prod/process-extract-text",
        {
            jobId: jobId
        }, "POST")
            .then(function (result) {
            //status: "IN_PROGRESS"|"SUCCEEDED"|"FAILED"|"PARTIAL_SUCCESS"
            setCache(retryCountKey, retryCount);
            switch (result.status) {
                case "IN_PROGRESS": {
                    if (retryCount > (maxRetryCount ? maxRetryCount : 5)) {
                        onErrorAPICall("ai", "RETRY_MAX_COUNT");
                        console.error(result.status);
                    }
                    else {
                        processAIExtractorJob(solution, record, requestType, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount);
                    }
                    break;
                }
                case "SUCCEEDED": {
                    record["".concat(requestType, "ResultForFields")] = map(filter(result.data, function (r) { return r.type == "field"; }), function (r, index) {
                        return __assign(__assign({}, r), { index: "field-".concat(index) });
                    });
                    record["".concat(requestType, "ResultForRows")] = map(filter(result.data, function (r) { return r.type == "row"; }), function (r, index) {
                        return __assign(__assign({}, r), { index: "row-".concat(index) });
                    });
                    //if it is task, we will need to use mapping to generate output
                    record["".concat(requestType, "Result")] =
                        processAIExtractorResult(requestType, record);
                    updateRecord(record);
                    onSuccessAPICall("ai");
                    if (onSuccessProcessAIExtractorJob) {
                        onSuccessProcessAIExtractorJob(record);
                    }
                    break;
                }
                default: {
                    onErrorAPICall("ai", result.status);
                    console.error(result.status);
                    break;
                }
            }
        })
            .catch(function (error) {
            setCache(retryCountKey, retryCount);
            onErrorAPICall("ai", error);
            console.error(error);
        });
    }, 1000 * (delaySeconds ? delaySeconds : 3));
};
export var startAIExtractorJob = function (solution, record, requestType, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount) {
    var jobId = record["".concat(requestType, "JobId")];
    if (jobId) {
        return processAIExtractorJob(solution, record, requestType, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount);
    }
    onStartAPICall("ai");
    var fileS3Name = getAIExtractorS3FileName(requestType, record);
    var fileId = getAIExtractorFileId(requestType, record);
    callApi(solution, getApiEndpoint(solution, "ai", "start-extract-text"), 
    // "http://localhost:3000/prod/start-extract-text",
    // `${solution.apis.ai}start-extract-text`,
    {
        fileName: fileS3Name,
        id: fileId
    }, "POST")
        .then(function (result) {
        record["".concat(requestType, "JobId")] = result.jobId;
        updateRecord(record);
        processAIExtractorJob(solution, record, requestType, result.jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount);
    })
        .catch(function (error) {
        onErrorAPICall("ai", error);
        console.error(error);
    });
};
//# sourceMappingURL=ai-extractor-util.js.map