//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, useEffect, useRef } from "react";
import SearchSection from "../search";
import { callApi } from "../../context/auth-call-api";
import SVG from "primeobjects-ui-web-basic/build/cjs/controls/svg";
import { useEnvStore } from "primeobjects-ui-web-store";
import { observer } from "mobx-react-lite";
import { COLORS, isNonEmptyString, newGuid, formatNumberBMK } from "primeobjects-helper-util";
import { InProgress } from "../../controls/in-progress";
import Button from "../../controls/action-button";
import { usePrimaryColor } from "primeobjects-helper-ui";
import Dropdown from "../../controls/antd/dropdown";
import Menu from "../../controls/antd/menu";
import Popover from "../../controls/antd/popover";
import { Parser } from "@json2csv/plainjs";
import { map, each, cloneDeep, isNumber, uniq, isNil } from "lodash";
import { getLocalStorage, setLocalStorage } from "primeobjects-ui-web-basic/build/cjs/util/local-storage";
import { useSolution } from "primeobjects-helper-ui/build/cjs/hooks/solution";
import { getApiEndpoint, getApiEndpointBase } from "../../util";
export var ListHeaderSection = observer(function (props) {
    var _a, _b;
    var entity = props.entity, cancelSearch = props.cancelSearch, showTags = props.showTags, hideSearchService = props.hideSearchService, queryResult = props.queryResult, apis = props.apis;
    var envStore = useEnvStore();
    var envData = JSON.parse(envStore.data);
    var searching = (_a = envData === null || envData === void 0 ? void 0 : envData.search) === null || _a === void 0 ? void 0 : _a.doing;
    var primaryColor = usePrimaryColor();
    var solution = useSolution();
    var createButtonText = props.createButtonText
        ? props.createButtonText
        : "New";
    var _c = useState(undefined), searchService = _c[0], setSearchService = _c[1];
    var hideCreateButton = entity.disableCreate == true ||
        props.hideCreateButton == true ||
        !props.onClickCreateButton;
    var exportProps = (_b = props.exportProps) !== null && _b !== void 0 ? _b : [
        "id|string|ID",
        "display|string|Display",
        "stateCode|int|State Code",
        "statusCode|int|Status Code",
        "createdOn|datetime|Created On",
        "modifiedOn|datetime|Modified On",
        "organizationId|string|Organization ID",
        "solutionId|string|Solution ID"
    ];
    var _d = useState(false), openCategories = _d[0], setOpenCategories = _d[1];
    var exportResult = useRef({
        count: 0,
        data: []
    });
    var exportQuery = useRef(undefined);
    var _e = useState(-1), exportCount = _e[0], setExportCount = _e[1];
    var abortController = useRef();
    var isExporting = exportCount >= 0;
    var query = queryResult === null || queryResult === void 0 ? void 0 : queryResult.query;
    var result = queryResult === null || queryResult === void 0 ? void 0 : queryResult.result;
    useEffect(function () {
        //check whether there is a local saved search service
        var localQueryService = getLocalStorage("query-service-".concat(query === null || query === void 0 ? void 0 : query.entityName));
        setSearchService(hideSearchService
            ? "hidden"
            : (query === null || query === void 0 ? void 0 : query.service) != "hidden" && localQueryService
                ? localQueryService
                : (query === null || query === void 0 ? void 0 : query.service)
                    ? query === null || query === void 0 ? void 0 : query.service
                    : "auto");
    }, [query]);
    var onSubmitSearch = function (keywords) {
        props.onSubmitSearch && props.onSubmitSearch(keywords);
    };
    var onRefresh = function () {
        props.onRefresh && props.onRefresh();
    };
    var getPropValue = function (record, name) {
        if (!name || !isNonEmptyString(name) || !record)
            return undefined;
        var layers = name.split(".");
        if (layers.length > 1) {
            var curName = layers.shift();
            if (curName) {
                var newRecord = record[curName];
                var newName = layers.join(".");
                return getPropValue(newRecord, newName);
            }
            else {
                return undefined;
            }
        }
        return record[name];
    };
    var onClickExport = function (all) {
        all ? onClickExportAll() : onClickExportCurrent();
    };
    var onClickExportAll = function () {
        try {
            if (query) {
                exportQuery.current = cloneDeep(query);
                exportQuery.current.name = "first";
                exportQuery.current.attributes = uniq(map(exportProps, function (p) {
                    return p.split("|")[0].split(".")[0];
                })).join(",");
                exportQuery.current.pageNumber = 1;
                exportQuery.current.pageSize = 50;
                queryExportAll(0);
            }
        }
        catch (err) {
            console.error(err);
        }
    };
    var queryExportAll = function (count) {
        var _a;
        var apiEndpoint = apis
            ? getApiEndpointBase(apis, "data", "query")
            : getApiEndpoint(solution, "data", "query", entity === null || entity === void 0 ? void 0 : entity.entityName);
        if (apiEndpoint) {
            (_a = abortController.current) === null || _a === void 0 ? void 0 : _a.abort();
            abortController.current = new AbortController();
            console.log({ count: count });
            setExportCount(count);
            callApi(solution, apiEndpoint, { query: exportQuery.current }, "post", {
                abortSignal: abortController.current.signal
            })
                .then(function (r) {
                var _a, _b;
                var newResult = { count: 0, data: [] };
                switch ((_a = exportQuery.current) === null || _a === void 0 ? void 0 : _a.name) {
                    case "more": {
                        newResult = {
                            count: exportResult.current.count + r.count,
                            continuation: r.continuation,
                            data: __spreadArray(__spreadArray([], exportResult.current.data, true), r.data, true)
                        };
                        break;
                    }
                    default: {
                        newResult = r;
                        break;
                    }
                }
                exportResult.current = newResult;
                if (isNumber(newResult.continuation) &&
                    ((_b = exportQuery.current) === null || _b === void 0 ? void 0 : _b.pageNumber)) {
                    exportQuery.current.name = "more";
                    exportQuery.current.pageNumber =
                        exportQuery.current.pageNumber + 1;
                    queryExportAll(newResult.count);
                }
                else {
                    setExportCount(-1);
                    exportData(newResult);
                }
            })
                .catch(function (ex) {
                console.error(ex);
                alert("Sorry, the export failed, please try again later.");
                setExportCount(-1);
                abortController.current = null;
            });
        }
    };
    var onClickExportCurrent = function () {
        exportData(result);
    };
    var exportData = function (result) {
        try {
            if (result) {
                var parser = new Parser();
                var data = map(result === null || result === void 0 ? void 0 : result.data, function (r) {
                    var newR = {};
                    each(exportProps, function (fname) {
                        var nameType = fname.split("|");
                        var type = nameType.length > 1 ? nameType[1] : "";
                        var name = nameType[0];
                        var label = nameType.length > 2 ? nameType[2] : name;
                        var value = getPropValue(r, name);
                        switch (type) {
                            case "datetime": {
                                newR[label] = !isNil(value)
                                    ? new Date(value)
                                    : "";
                                break;
                            }
                            case "int": {
                                newR[label] = !isNil(value)
                                    ? parseInt(value)
                                    : "";
                                break;
                            }
                            case "percent": {
                                newR[label] = !isNil(value)
                                    ? parseFloat(value).toFixed(2)
                                    : "";
                                break;
                            }
                            case "money": {
                                newR[label] = !isNil(value)
                                    ? parseFloat(value).toFixed(4)
                                    : "";
                                break;
                            }
                            case "number":
                            case "float": {
                                newR[label] = !isNil(value)
                                    ? parseFloat(value)
                                    : "";
                                break;
                            }
                            default: {
                                newR[label] = !isNil(value) ? value : "";
                                break;
                            }
                        }
                        // if (
                        //     isNumber(v) &&
                        //     isDate(new Date(v)) &&
                        //     new Date(v).getTime() -
                        //         new Date("1971-1-1").getTime() >
                        //         0
                        // ) {
                        //     newR[fname] = new Date(v);
                        // } else {
                        //     newR[fname] = v;
                        // }
                    });
                    return newR;
                });
                var csv = parser.parse(data);
                console.log({ csv: csv });
                var element = document.createElement("a");
                element.setAttribute("href", "data:text/plain;charset=utf-8," +
                    encodeURIComponent("".concat(csv)));
                element.setAttribute("download", "".concat(entity.entityName.toLowerCase()).concat(entity.entityType
                    ? "-".concat(entity.entityType.toLowerCase())
                    : "", "-").concat(Date.now(), ".csv"));
                element.style.display = "none";
                document.body.appendChild(element);
                element.click();
                document.body.removeChild(element);
            }
        }
        catch (err) {
            console.error(err);
        }
    };
    var onClickToggleTagsButton = function () {
        props.onClickToggleTagsButton && props.onClickToggleTagsButton();
    };
    var onClickFilterButton = function () { };
    var onClickCreateButton = function () {
        props.onClickCreateButton &&
            props.onClickCreateButton({
                id: newGuid(),
                entityName: entity.entityName,
                entityType: entity.entityType
            });
    };
    var onClickSearchService = function (service) {
        setLocalStorage("query-service-".concat(query === null || query === void 0 ? void 0 : query.entityName), service);
        setSearchService(service);
        props.onRefresh && props.onRefresh();
    };
    var renderCurrentSearchServiceName = function () {
        switch (searchService) {
            case "database": {
                return "Simple search engine";
            }
            case "elastic": {
                return "Full-text search engine";
            }
            default: {
                return "Auto search engine";
            }
        }
    };
    return (React.createElement("div", { className: "w-full flex flex-row py-4 items-center relative bg-white" },
        !props.hideEntityIcon && (React.createElement("div", { className: "flex ml-4" },
            React.createElement(SVG, { src: entity.icon, size: 32, color: COLORS.GRAY["900"] }))),
        props.onSubmitSearch &&
            !props.hideSearch &&
            (isExporting ? (React.createElement("div", { className: "w-full p-1" })) : (React.createElement(React.Fragment, null,
                React.createElement(SearchSection, { searching: searching, onSubmit: onSubmitSearch, placeholder: entity.searchInputPlaceholder
                        ? entity.searchInputPlaceholder
                        : "Search ".concat(entity.uiCollectionName, " ..."), wrapperStyle: searchService == "hidden"
                        ? { height: 32 }
                        : { height: 20 }, wrapperClassName: searchService == "hidden" ? "px-2" : "px-2 pt-2", inputClassName: "max-w-xs", cancelSearch: cancelSearch }),
                searchService != "hidden" && (React.createElement("div", { className: "absolute cursor-pointer", style: { top: 8, left: 56 } },
                    React.createElement("div", { style: { width: 150 } },
                        React.createElement(Dropdown, { trigger: ["click"], placement: "top", overlay: React.createElement(Menu, null,
                                React.createElement(Menu.Item, { onClick: function () {
                                        return onClickSearchService("auto");
                                    } }, "Auto search engine"),
                                React.createElement(Menu.Item, { onClick: function () {
                                        return onClickSearchService("database");
                                    } }, "Simple search engine"),
                                React.createElement(Menu.Item, { onClick: function () {
                                        return onClickSearchService("elastic");
                                    } }, "Full-text search engine")) },
                            React.createElement("div", { className: "w-full flex" },
                                React.createElement("div", { className: "text-xs" }, renderCurrentSearchServiceName()),
                                React.createElement(SVG, { style: { margin: 4 }, src: "/icons/platform/down.svg", size: 10, color: COLORS.GRAY["900"] }))))))))),
        React.createElement("div", { className: "flex mr-4" },
            props.hideCategoriesButton != true &&
                !isExporting &&
                props.renderCategories && (React.createElement(Popover, { placement: "bottomRight", open: openCategories, onOpenChange: function (newOpen) {
                    setOpenCategories(newOpen);
                }, trigger: "click", content: React.createElement("div", { style: { width: 350 } }, props.renderCategories(function () {
                    setOpenCategories(false);
                })) },
                React.createElement(Button, { id: "list-header-categories", iconSrc: "/icons/platform/categories.svg" }))),
            (exportProps === null || exportProps === void 0 ? void 0 : exportProps.length) > 0 &&
                !props.hideExportButton &&
                (isExporting ? (React.createElement("div", { className: "min-w-32 flex flex-row" },
                    React.createElement("div", { className: "min-w-18" },
                        React.createElement(InProgress, null)),
                    React.createElement("div", { className: "leading-none self-center w-full text-sm" }, "Exporting"),
                    exportCount > 0 && (React.createElement("div", { className: "ml-1 leading-none self-center w-full text-sm" }, exportCount < 1000
                        ? exportCount
                        : formatNumberBMK(exportCount))))) : (React.createElement(Dropdown, { trigger: ["click"], overlay: React.createElement(Menu, null,
                        React.createElement(Menu.Item, { key: "export-current", className: "w-full", onClick: function () {
                                onClickExport(false);
                            } },
                            React.createElement("span", null, "Export current records")),
                        React.createElement(Menu.Item, { key: "export-all", className: "w-full", onClick: function () {
                                onClickExport(true);
                            } },
                            React.createElement("span", null, "Export all records"))), placement: "bottomLeft" },
                    React.createElement(Button, { id: "list-header-export-csv", iconSrc: "/icons/platform/export-csv.svg" })))),
            props.onRefresh && !isExporting && (React.createElement(Button, { id: "list-header-refresh", onClick: onRefresh, iconSrc: "/icons/platform/refresh.svg" })),
            props.onClickToggleTagsButton && !isExporting && (React.createElement(Button, { id: "list-header-filter", onClick: onClickToggleTagsButton, iconSrc: "/icons/platform/tags.svg", tooltipText: "Toggle tags", iconColor: showTags ? primaryColor["600"] : undefined })),
            props.onClickFilterButton && !isExporting && (React.createElement(Button, { id: "list-header-filter", onClick: onClickFilterButton, iconSrc: "/icons/platform/filter.svg" })),
            !hideCreateButton && !isExporting && (React.createElement(Button, { id: "list-header-new", onClick: onClickCreateButton, wrapperClassName: "bg-green-500 hover:bg-green-600", text: createButtonText, textColor: COLORS.WHITE })))));
});
export default ListHeaderSection;
//# sourceMappingURL=header.js.map